@@ -1,4 +1,4 @@
-<TeXmacs|1.99.2>
+<TeXmacs|1.99.11>
 
 <style|<tuple|source|std-pattern>>
 
@@ -51,6 +51,13 @@
 
   <assign|noteref-sep|<macro|>>
 
+  <assign|cite-website|<macro|<localize|This document has been written using>
+  GNU <TeXmacs><if|<equal|<value|language>|french>| ; |; ><localize|see>
+  <hlink|<with|font-family|tt|www.texmacs.org>|http://www.texmacs.org>.>>
+
+  <assign|cite-TeXmacs|<xmacro|keys|<localize|This document has been written
+  using> GNU <TeXmacs><nbsp><map-args|identity|cite|keys>.>>
+
   <\active*>
     <\src-comment>
       DRD properties of tags with title and author data.
@@ -87,6 +94,8 @@
 
   <drd-props|author-affiliation|border|no>
 
+  <drd-props|author-name-affiliation|border|no>
+
   <drd-props|author-email|border|no>
 
   <drd-props|author-homepage|border|no>
@@ -123,6 +132,14 @@
 
   <assign|doc-make-title|<macro|body|<surround||<vspace|2fn>|<doc-title-block|<arg|body>>>>>
 
+  <assign|doc-make-rich-title|<\macro|notes|body>
+    <\surround||<arg|notes>>
+      <\doc-make-title>
+        <arg|body>
+      </doc-make-title>
+    </surround>
+  </macro>>
+
   <assign|doc-title|<macro|x|<\surround|<vspace*|0.5fn>|<vspace|0.5fn>>
     <doc-title-block|<font-magnify|1.682|<with|math-font-series|bold|font-series|bold|<arg|x>>>>
   </surround>>>
@@ -149,7 +166,7 @@
 
   <assign|doc-footnote-text|<macro|sym|id|body|<custom-footnote-text|<arg|sym>|<arg|id>|<arg|body>>>>
 
-  <assign|doc-data|<xmacro|args|<extern|doc-data|<quote-arg|args>>>>
+  <assign|doc-data|<xmacro|args|<extern|doc-data|<quote-arg|args>|>>>
 
   <\active*>
     <\src-comment>
@@ -173,6 +190,9 @@
     <surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<arg|address>>>
   </macro>>
 
+  <assign|author-name-affiliation|<macro|author|address|<surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<name|<arg|author>>,
+  <arg|address>>>>>
+
   <assign|author-affiliation-note|<\macro|sym|id|address>
     <author-affiliation|<\doc-note-text|<arg|sym>|<arg|id>>
       <arg|address>
@@ -180,16 +200,16 @@
   </macro>>
 
   <assign|author-email|<macro|email|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<email-text><localize|:>
-  ><with|font-family|tt|<arg|email>>>>>>
+  ><with|font-family|tt|language|verbatim|<arg|email>>>>>>
 
   <assign|author-email-note|<macro|sym|id|email|<doc-author-block|<doc-note-text|<arg|sym>|<arg|id>|<style-with|src-compact|none|<with|font-shape|italic|<email-text><localize|:>
-  ><with|font-family|tt|<arg|email>>>>>>>
+  ><with|font-family|tt|language|verbatim|<arg|email>>>>>>>
 
   <assign|author-homepage|<macro|homepage|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<homepage-text><localize|:>
-  ><with|font-family|tt|<arg|homepage>>>>>>
+  ><with|font-family|tt|language|verbatim|<arg|homepage>>>>>>
 
   <assign|author-homepage-note|<macro|sym|id|homepage|<doc-author-block|<doc-note-text|<arg|sym>|<arg|id>|<style-with|src-compact|none|<with|font-shape|italic|<homepage-text><localize|:>
-  ><with|font-family|tt|<arg|homepage>>>>>>>
+  ><with|font-family|tt|language|verbatim|<arg|homepage>>>>>>>
 
   <assign|author-misc|<\macro|body>
     <surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<arg|body>>>
