@@ -24,7 +24,10 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (beamer-themes)
-  (list "bluish" "ice" "metal" "reddish" "ridged-paper"))
+  (list "blackboard" "bluish" "boring-white" "dark-vador" "granite"
+        "ice" "manila-paper" "metal" "pale-blue" "parchment"
+        "pine" "reddish" "ridged-paper" "rough-paper"
+        "xperiment"))
 
 (tm-define (current-beamer-theme)
   (with l (get-style-list)
@@ -964,10 +967,27 @@
        (tree-is? (tree-ref t :up) 'document)
        (list-context? (tree-ref t :up :up))))
 
+(tm-define (make-unroll switch-tag)
+  (if (and (selection-active?)
+           (tree-in? (selection-tree) (list-tag-list))
+           (tree-is? (selection-tree) 0 'document))
+      (let* ((sel (selection-tree))
+             (list-tag (tree-label sel))
+             (items (tree-children (tree-ref sel 0)))
+             (fun (lambda (c) `(shown (document ,c))))
+             (unroll `(,switch-tag ,@(map fun items)))
+             (new-list `(,list-tag (document ,unroll))))
+        (clipboard-cut "dummy")
+        (insert new-list))
+      (make-switch switch-tag)))
+
 (tm-define (make-switch-list switch-tag list-tag)
-  (make list-tag)
-  (make-switch switch-tag)
-  (make-item))
+  (with flag? (and (selection-active-non-small?)
+                   (in? list-tag (description-tag-list)))
+    (wrap-selection-any
+      (make list-tag)
+      (make-switch switch-tag)      
+      (if flag? (insert '(item* "")) (make-item)))))
 
 (tm-define (kbd-enter t shift?)
   (:require (switch-list-context? t))
